#include <stdlib.h>
#include <string.h>

#include "common.h"

/*
The word in-place appears a few times here. It just means that the functions alter the image array directly
rather than, say, making a new array where the changes are implemented.

The usual indexing applies, [i][j] becomes [n * i + j].

Besides that, you shouldn't have any trouble understanding how each function works.
*/

/*
This is an in-place implementation of a vertical flip. We can swap a whole line at a time. Each line
has coordinates [i][0:j:n], 0 <= i < m, and we swap line [i] with line [m - i], 0 <= i < m / 2.
*/
void flip_vertical(image *img) {

    char *tmp = (char *)malloc(img->n * sizeof(char));

    for (int i = 0; i < img->m / 2; i ++) {

        memcpy(tmp, &img->image_chars[img->n * i], img->n * sizeof(char));
        memcpy(&img->image_chars[img->n * i], &img->image_chars[img->n * (img->m - 1 - i)], img->n * sizeof(char));
        memcpy(&img->image_chars[img->n * (img->m - 1 - i)], tmp, img->n * sizeof(char));
    }

    free(tmp);
}

/*
Flips the image horizontally, also done in-place. Flips a single character at a time. Specifically
we swap character [i][j] with character [i][n - j], 0 <= i < m, 0 <= i < n / 2.
*/
void flip_horizontal(image *img) {

    unsigned char tmp;

    for (int i = 0; i < img->m; i ++) {
        for (int j = 0; j < img->n / 2; j ++) {

            tmp = img->image_chars[img->n * i + j];
            img->image_chars[img->n * i + j] = img->image_chars[img->n * (i + 1) - 1 - j];
            img->image_chars[img->n * (i + 1) - 1 - j] = tmp;

        }
    }
}

unsigned char compute_average(image *img) {

    float sum = 0.0;

    for (int i = 0; i < img->m * img->n; i ++) sum += (float)img->image_chars[i];

    // Add .5 to the result to ensure rounding instead of truncating.
    return (unsigned char) ((sum / (float)img->m * img->n) + 0.5);
}